@@ -4,9 +4,6 @@
 # This conveniently wraps melting and casting a data frame into
 # one step. 
 #
-# Todo: take advantage of reshape formula to reduce number of 
-# columns in molten data frame to save time/spcae.
-# 
 # @arguments Data set to melt
 # @arguments Casting formula, see \link{cast} for specifics
 # @arguments Other arguments passed to \link{cast}
@@ -14,9 +11,11 @@
 # @arguments Measured variables. If blank, will use all non id.var variables
 # @keyword manip
 # @seealso \url{http://had.co.nz/reshape/}
+#X recast(french_fries, time ~ variable, id.var=1:4)
 recast <- function(data, formula, ..., id.var, measure.var) {
-	var <- cast_parse_formula(deparse(substitute(formula)), names(data))
+	#var <- unlist(cast_parse_formula(deparse(substitute(formula)), names(data)))
+	#used <- intersect(names(data), union(var, measure.var))
 
 	molten <- melt(data, id.var, measure.var)
-	cast(molten, formula, ...)
+	cast(molten, deparse(substitute(formula)), ...)
 }
